<?php

namespace App\Http\Controllers;

use App\AdminOtherAccount;
use App\AdminReceiptVoucher;
use App\AdminSalesInvoice;
use App\Company;
use App\Customer;
use App\Expense;
use App\ReceiptVoucherExpense;
use App\SaleInvoice;
use App\StaffAndSalesman;
use App\Supplier;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class AdminReceiptVoucherController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $receipts = AdminReceiptVoucher::with('expense.account')
        ->whereIn('id', function ($query) {
            $query->selectRaw('MIN(id)')
                ->from('admin_receipt_vouchers')
                ->groupBy('invoice_no');
        })
        ->where('branch_id', session()->get('selected_branch'))
        ->withTrashed()
        ->get();

        return view('crusher.trans_receipt_voucher.index', compact('receipts'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $salesInvoices = AdminSalesInvoice::where('branch_id', session()->get('selected_branch'))->get();
        // dd($salesInvoices);
        $currentDate = Carbon::now()->format('Ymd');

        // Get the latest quotation number for the branch
        $latestReceipts = AdminReceiptVoucher::where('branch_id', session()->get('selected_branch'))
            ->latest('id')->first();

        // Calculate the new quotation number
        if ($latestReceipts) {
            $lastReceipt = explode('-', $latestReceipts->invoice_no)[1];
            $newReceipt = "RV-".$currentDate . '-' . str_pad((int)$lastReceipt + 1, 3, '0', STR_PAD_LEFT);
        } else {
            $newReceipt = "RV-".$currentDate . '-001';
        }

        // Retrieve customers and include both 'account_name' and 'table_name' attributes
        $customers = Customer::where('branch_id', session()->get('selected_branch'))
        ->select('*', 'name_of_organisation as account_name')
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Customer');
        });

        // Retrieve suppliers and include both 'account_name' and 'table_name' attributes
        $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        ->select('*', 'name_of_organisation as account_name')
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Supplier');
        });

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
                $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        });    

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });

        // Retrieve other accounts and set 'table_name' to the fully-qualified model class name
        $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        // Concatenate all collections into a single collection
        $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);

        return view('crusher.trans_receipt_voucher.create', compact('newReceipt', 'customers', 'accounts', 'salesInvoices'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        try {
            $validatedData = $request->validate([
                'date' => 'nullable|date',
                'invoice_no' => 'nullable|string',
                'payment_method' => 'required|in:cash,bank',
                // 'bank_id' => 'nullable|required_if:payment_method,bank|exists:banks,id',
                'bank_id' => 'nullable|required_if:payment_method,bank',
                'cheaque_no' => 'nullable|required_if:payment_method,bank|string',
                'cheaque_date' => 'nullable|required_if:payment_method,bank|date',
                'bank_name' => 'nullable|required_if:payment_method,bank|string',
                'account_id' => 'nullable|array',
                'account_id.*' => 'nullable',
                'account_type' => 'nullable|array',
                'account_type.*' => 'nullable',
                'amount' => 'nullable|array',
                'amount.*' => 'nullable|numeric',
                'bill_wise_id' => 'nullable|array',
                // 'bill_wise_id.*' => 'nullable|exists:billwise,id',
                'bill_wise_id.*' => 'nullable',
                'inovice_id' => 'nullable|array',
                'inovice_id.*' => 'nullable',
                // 'invoice_id.*' => 'nullable|exists:invoices,id',
            ]);

            $branch_id = session()->get('selected_branch');

            $data = [
                'invoice_no' => $validatedData['invoice_no']??(NULL),
                'date' => $validatedData['date']??(NULL),
                'payment_method' => $validatedData['payment_method']??(NULL),
                'bank_id' => $validatedData['bank_id']??(NULL),
                'cheaque_no' => $validatedData['cheaque_no']??(NULL),
                'cheaque_date' => $validatedData['cheaque_date']??(NULL),
                'bank_name' => $validatedData['bank_name']??(NULL),
                'branch_id' => $branch_id,
            ];

                $receiptVoucher = AdminReceiptVoucher::create($data);
            for ($j = 0; $j < count($validatedData['account_id']); $j++) {
                ReceiptVoucherExpense::create([
                    'receipt_id' => $receiptVoucher->id,
                    'account_id' => $validatedData['account_id'][$j]??(NULL),
                    'account_type' => $validatedData['account_type'][$j]??(NULL),
                    'amount' => $validatedData['amount'][$j]??(NULL),
                    'bill_wise_id' => $validatedData['bill_wise_id'][$j]??(NULL),
                    'inovice_id' => $validatedData['bill_wise_id'][$j]=='BillWise'?$validatedData['inovice_id'][$j]:(NULL),
                ]);
            }

            flash()->addSuccess('Record Created');
            return redirect()->route('crusher.receipt_vouchers.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function show($receiptVoucher)
    {
        $receiptVoucher = AdminReceiptVoucher::withTrashed()->find($receiptVoucher);
        $receiptVoucher = $receiptVoucher->withTrashed()->with('expense.account')->where('invoice_no', $receiptVoucher->invoice_no)->get();
        $company = Company::where('user_id', auth()->id())->first();
        return view('crusher.trans_receipt_voucher.view', compact('receiptVoucher', 'company'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\AdminReceiptVoucher  $adminReceiptVoucher
     * @return \Illuminate\Http\Response
     */
    public function edit(AdminReceiptVoucher $receiptVoucher)
    {
        // Retrieve customers and include both 'account_name' and 'table_name' attributes
        $customers = Customer::where('branch_id', session()->get('selected_branch'))
        ->select('*', 'name_of_organisation as account_name')
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Customer');
        });

        // Retrieve suppliers and include both 'account_name' and 'table_name' attributes
        $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        ->select('*', 'name_of_organisation as account_name')
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Supplier');
        });

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
                $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        });    

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });

        // Retrieve other accounts and set 'table_name' to the fully-qualified model class name
        $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        // Concatenate all collections into a single collection
        $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);
;
        $receiptVouchers = $receiptVoucher->with('expense.account')->where('invoice_no', $receiptVoucher->invoice_no)->get();
        $salesInvoices = AdminSalesInvoice::where('branch_id', session()->get('selected_branch'))->get();

        return view('crusher.trans_receipt_voucher.edit', compact('accounts', 'receiptVouchers', 'salesInvoices'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\AdminReceiptVoucher  $adminReceiptVoucher
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, AdminReceiptVoucher $receiptVoucher)
    {
        try {
            $validatedData = $request->validate([
                'date' => 'nullable|date',
                'invoice_no' => 'nullable|string',
                'payment_method' => 'required|in:cash,bank',
                // 'bank_id' => 'nullable|required_if:payment_method,bank|exists:banks,id',
                'bank_id' => 'nullable|required_if:payment_method,bank',
                'cheaque_no' => 'nullable|required_if:payment_method,bank|string',
                'cheaque_date' => 'nullable|required_if:payment_method,bank|date',
                'bank_name' => 'nullable|required_if:payment_method,bank|string',
                'account_id' => 'nullable|array',
                'account_id.*' => 'nullable',
                'account_type' => 'nullable|array',
                'account_type.*' => 'nullable',
                'amount' => 'nullable|array',
                'amount.*' => 'nullable|numeric',
                'bill_wise_id' => 'nullable|array',
                // 'bill_wise_id.*' => 'nullable|exists:billwise,id',
                'bill_wise_id.*' => 'nullable',
                'inovice_id' => 'nullable|array',
                'inovice_id.*' => 'nullable',
                // 'invoice_id.*' => 'nullable|exists:invoices,id',
            ]);

            $branch_id = session()->get('selected_branch');

            $data = [
                'invoice_no' => $validatedData['invoice_no']??'',
                'date' => $validatedData['date']??'',
                'payment_method' => $validatedData['payment_method']??'',
                'bank_id' => $validatedData['bank_id']??'',
                'cheaque_no' => $validatedData['cheaque_no']??'',
                'cheaque_date' => $validatedData['cheaque_date']??"",
                'bank_name' => $validatedData['bank_name']??"",
                'branch_id' => $branch_id,
            ];
            $receiptVoucher->with('expense.account')->where('invoice_no', $receiptVoucher->invoice_no)->delete();


            $receiptVoucher = AdminReceiptVoucher::create($data);
            ReceiptVoucherExpense::where('receipt_id', $receiptVoucher->id)->delete();
            for ($j = 0; $j < count($validatedData['account_id']); $j++) {
                ReceiptVoucherExpense::create([
                    'receipt_id' => $receiptVoucher->id,
                    'account_id' => $validatedData['account_id'][$j]??'',
                    'account_type' => $validatedData['account_type'][$j]??'',
                    'amount' => $validatedData['amount'][$j]??'',
                    'bill_wise_id' => $validatedData['bill_wise_id'][$j]??'',
                    'inovice_id' => $validatedData['inovice_id'][$j]??'',
                ]);
            }

            flash()->addSuccess('Record Updated');
            return redirect()->route('crusher.receipt_vouchers.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\AdminReceiptVoucher  $adminReceiptVoucher
     * @return \Illuminate\Http\Response
     */
    public function destroy(AdminReceiptVoucher $adminReceiptVoucher)
    {
        //
    }

    public function softDelete($receiptVoucher)
    {
        $receiptVoucher = AdminReceiptVoucher::findOrFail($receiptVoucher);
        $receiptVoucher->where('invoice_no', $receiptVoucher->invoice_no)->delete();
        flash()->addSuccess('Success');
        return redirect()->route('crusher.receipt_vouchers.index');
    }
}
