<?php

namespace App\Http\Controllers;

use App\Branches;
use App\ClassOfAccount;
use App\Country;
use App\Http\Requests\StoreStaffRequest;
use App\Http\Requests\UpdateStaffRequest;
use App\StaffAndSalesman;
use Illuminate\Support\Facades\Storage;

class StaffMasterController extends Controller
{
    public function index()
    {
        if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'staff-and-salesmen-account-master')) return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);
        $staffs = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))->get();
        return view('crusher.staff_master.index', compact('staffs'));
    }

    public function create()
    {
        if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'staff-and-salesmen-account-master')) return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);

        $countries = Country::all();
        $classOfAccount = ClassOfAccount::all();
        // $maintainAccount = Maintain::all();
        return view('crusher.staff_master.create', compact('countries', 'classOfAccount'));
    }

    public function store(StoreStaffRequest $request)
    {
        if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'staff-and-salesmen-account-master')) return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);
        try {

            $validatedData = $request->validated();
            $validatedData['branch_id'] = session()->get('selected_branch');
    
            // Handle file uploads for documents
            for ($i = 1; $i <= 7; $i++) {
                if ($request->hasFile("doc_file_$i")) {
                    $file = $request->file("doc_file_$i");
                    $fileName = time() . '_doc_' . $i . '.' . $file->getClientOriginalExtension();
                    $filePath = $file->storeAs('uploads/admin/staff_master/docs/', $fileName, 'public');
                    $validatedData["doc_file_$i"] = $filePath;
                }
            }

        // Create a new staff record
        StaffAndSalesman::create($validatedData);
    
            flash()->addSuccess('Record Created');
            return redirect()->route('crusher.staffs.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function edit($staff)
    {
        if(!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'staff-and-salesmen-account-master')) return response()->json(['status'=>false, 'error'=>'Unauthorized'], 403);
        $countries = Country::all();
        $classOfAccount = ClassOfAccount::all();
        $staff = StaffAndSalesman::findOrFail($staff);
        return view('crusher.staff_master.edit', compact('countries', 'classOfAccount', 'staff'));
    }

    public function update(UpdateStaffRequest $request, StaffAndSalesman $staff)
    {
        if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'staff-and-salesmen-account-master')) {
            return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);
        }

        try {
            $validatedData = $request->validated();
            $validatedData['branch_id'] = session()->get('selected_branch');

            // Handle file uploads for authorized signatory signature samples
            for ($i = 1; $i <= 3; $i++) {
                if ($request->hasFile("authorised_sign_{$i}_sign_sample")) {
                    $file = $request->file("authorised_sign_{$i}_sign_sample");
                    $fileName = time() . '_auth_sign_' . $i . '.' . $file->getClientOriginalExtension();
                    $filePath = $file->storeAs('uploads/admin/staff_master/authorised_docs/', $fileName, 'public');
                    $validatedData["authorised_sign_{$i}_sign_sample"] = $filePath;

                    // Delete existing file if it exists
                    if (!empty($staff["authorised_sign_{$i}_sign_sample"])) {
                        Storage::disk('public')->delete($staff["authorised_sign_{$i}_sign_sample"]);
                    }
                }
            }

            // Handle file uploads for documents
            for ($i = 1; $i <= 7; $i++) {
                if ($request->hasFile("doc_file_$i")) {
                    $file = $request->file("doc_file_$i");
                    $fileName = time() . '_doc_' . $i . '.' . $file->getClientOriginalExtension();
                    $filePath = $file->storeAs('uploads/admin/staff_master/docs/', $fileName, 'public');
                    $validatedData["doc_file_$i"] = $filePath;

                    // Delete existing file if it exists
                    if (!empty($staff["doc_file_$i"])) {
                        Storage::disk('public')->delete($staff["doc_file_$i"]);
                    }
                }
            }

            // Update the staff record
            $staff->update($validatedData);

            flash()->addSuccess('Record Updated');
            return redirect()->route('crusher.staffs.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function softDelete($staff)
    {
        $staff = StaffAndSalesman::findOrFail($staff);
        $staff->delete();
        flash()->addSuccess('Success');
        return redirect()->route('crusher.staffs.index');
    }

    function show($staff)
    {
        if(!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'staff-and-salesmen-account-master')) return response()->json(['status'=>false, 'error'=>'Unauthorized'], 403);
        $staff = StaffAndSalesman::with('country', 'classOfAccount')->findOrFail($staff);
        return view('crusher.staff_master.view', compact('staff'));
    }
}
