<?php

namespace App\Http\Middleware;

use App\Branches;
use Carbon\Carbon;
use Closure;

class BranchExpireCheckMiddleware
{
    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure  $next
     * @return mixed
     */
    public function handle($request, Closure $next)
    {
        $selectedBranchId = session('selected_branch');
        $expire = false;
        $leniencyExpire = false;
        if (!empty($selectedBranchId)) 
        {
            $branch = Branches::find($selectedBranchId);
            $now = Carbon::parse(now())->format('Y-m-d');
            $branchExpireDate = Carbon::parse($branch->expiry_date);
            $leniency = $branch->leniency;
            if($branchExpireDate->addDays($leniency)->lte($now))
            {
                $expire = true;
                if (!$branch->vouchers->isEmpty()) 
                {
                    foreach ($branch->vouchers as $voucher) 
                    {
                        if ($voucher->created_at->gte($branchExpireDate)) 
                        {
                            $expire = false;
                        }
                    }
                }
            }
            if ($expire) {
                flash()->addError('Your license is expire');
                return back();
            }

            if (!$branch->status)
            {
                flash()->addError('Branch Block Contact Admin');
                return back();
            }
        }
        else
        {
            flash()->addError('Please select branch first');
            return back();
        }
        return $next($request);
    }
}
