<?php

namespace App\Http\Controllers;

use App\AdminOtherAccount;
use App\AdminTermsAndCondition;
use App\Company;
use App\Customer;
use App\DeliveryNote;
use App\Expense;
use App\ItemName;
use App\Quotation;
use App\StaffAndSalesman;
use App\Supplier;
use App\OtherAccountBuyer;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class DeliveryNoteController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $deliveryNotes = DeliveryNote::with('item', 'account')
        ->whereIn('id', function ($query) {
            $query->selectRaw('MIN(id)')
                // ->from('delivery_notes')
                ->groupBy('do_no');
        })
        ->where('branch_id', session()->get('selected_branch'))
        ->withTrashed()
        ->orderBy('id', 'desc')->get();
        return view('crusher.trans_delivery_note.index', compact('deliveryNotes'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        // Retrieve customers and include both 'account_name' and 'table_name' attributes
        $customers = Customer::where('branch_id', session()->get('selected_branch'))
        ->select('*', 'name_of_organisation as account_name')
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Customer');
        });

        // Retrieve suppliers and include both 'account_name' and 'table_name' attributes
        $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        ->select('*', 'name_of_organisation as account_name')
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Supplier');
        });

        $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        });

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });

        // Retrieve other accounts and set 'table_name' to the fully-qualified model class name
        $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        // Concatenate all collections into a single collection
        $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);


        $items = ItemName::where('branch_id', session()->get('selected_branch'))->get();

        $currentDate = Carbon::now()->format('Ymd');
        // Get the latest quotation number for the branch
        $latestQuotation = Quotation::where('branch_id', session()->get('selected_branch'))
            ->latest('id')->first();

        // Calculate the new quotation number
        if ($latestQuotation) {
            $lastQuotationNumber = explode('-', $latestQuotation->quotation_no)[1];
            $newQuotationNumber = $currentDate . '-' . str_pad((int)$lastQuotationNumber + 1, 4, '0', STR_PAD_LEFT);
        } else {
            $newQuotationNumber = $currentDate . '-0001';
        }

        // Get the latest quotation number for the branch
        $latestDo = DeliveryNote::where('branch_id', session()->get('selected_branch'))
        ->latest('id')->first();

        // Calculate the new quotation number
        if ($latestDo) {
            $lastNumber = (int)explode('-', $latestDo->do_no)[1];
            $newDoNumber = $currentDate . '-' . str_pad($lastNumber + 1, 4, '0', STR_PAD_LEFT);
        } else {
            $newDoNumber = $currentDate . '-0001';
        }


        $quotationNo = request()->input('quotation_no');

        if ($quotationNo) {
            $quotations = Quotation::where('quotation_no',$quotationNo)->get();
        } else {
            $quotations = [];
        }

        $debitNo = request()->input('pi_no');

        if ($debitNo) {
            $debit = AdminPurchaesInvoice::where('po_no',$debitNo)->get();
        } else {
            $debit = [];
        }

        $tnc = AdminTermsAndCondition::where(['branch_id'=>session()->get('selected_branch'), 'type'=>'delivery_note'])->first();

        return view('crusher.trans_delivery_note.create', compact('tnc','newQuotationNumber', 'newDoNumber', 'customers', 'items', 'quotations', 'accounts','debit'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        try {
            $validatedData = $request->validate([
                'quotation_no' => 'nullable|string',
                'invoice_no' => 'nullable|string',
                'do_no' => 'nullable|string',
                'date' => 'nullable|date',
                'account_id' => 'nullable',
                'account_type' => 'nullable',
                'item_id.*' => 'nullable|exists:item_names,id',
                'item_description.*' => 'nullable|string',
                'quantity.*' => 'nullable|numeric',
                'total_quantity' => 'nullable|numeric',
                'narrations' => 'nullable|string',
                'name' => 'nullable|string',
                'phone' => 'nullable|string',
                'email' => 'nullable|string',
                'vat_number' => 'nullable|string',
                'address' => 'nullable|string',
                'type' => 'nullable|string',
            ]);

            $branch_id = session()->get('selected_branch');

             // Create an array to store buyer data
             $buyerData = [
                'name' => $validatedData['name'],
                'phone' => $validatedData['phone'],
                'email' => $validatedData['email'],
                'vat_number' => $validatedData['vat_number'],
                'address' => $validatedData['address'],
                'invoice_no' => $validatedData['invoice_no'],
                'type' => $validatedData['type'],
            ];

            // Check if buyer_account_id is present and create or update the buyer record
            if ($request->has('buyer_account_id')) {
                OtherAccountBuyer::updateOrCreate(['id' => $request->input('buyer_account_id')], $buyerData);
            } else {
                $buyer = OtherAccountBuyer::create($buyerData);
                $request->merge(['buyer_account_id' => $buyer->id]);
            }


            $quotationNo = $request->input('quotation_no'); // Quotation number from URL

            for ($i = 0; $i < count($validatedData['quantity']); $i++) {
                $deliveryNoteData = [
                    'do_no' => $validatedData['do_no'],
                    'total_quantity' => $validatedData['total_quantity'],
                    'narrations' => $validatedData['narrations'],
                    'quotation_no' => $quotationNo ?: $validatedData['quotation_no'], // Use URL if available, otherwise use form input
                    'date' => $validatedData['date'],
                    'account_id' => $validatedData['account_id'],
                    'account_type' => $validatedData['account_type'],
                    'item_id' => $validatedData['item_id'][$i],
                    'item_description' => $validatedData['item_description'][$i],
                    'quantity' => $validatedData['quantity'][$i],
                    'branch_id' => $branch_id,
                ];

                $do = DeliveryNote::create($deliveryNoteData);
            }

            flash()->addSuccess('Record Created');
            return redirect()->route('crusher.delivery_notes.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Quotation  $quotation
     * @return \Illuminate\Http\Response
     */
    public function show($deliveryNote)
    {
        $deliveryNote = DeliveryNote::withTrashed()->find($deliveryNote);
        $deliveryNote = $deliveryNote->withTrashed()->with('item', 'account', 'buyer')->where('do_no', $deliveryNote->do_no)->get();
        // dd($deliveryNote);
        $company = Company::where('user_id', auth()->id())->first();
        $tnc = AdminTermsAndCondition::where(['branch_id'=>session()->get('selected_branch'), 'type'=>'delivery_note'])->first();
        return view('crusher.trans_delivery_note.view', compact('deliveryNote', 'company', 'tnc'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Quotation  $quotation
     * @return \Illuminate\Http\Response
     */
    public function edit($deliveryNote)
    {
        // Retrieve customers and include both 'account_name' and 'table_name' attributes
        $customers = Customer::where('branch_id', session()->get('selected_branch'))
        ->select('*', 'name_of_organisation as account_name')
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Customer');
        });

        // Retrieve suppliers and include both 'account_name' and 'table_name' attributes
        $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        ->select('*', 'name_of_organisation as account_name')
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Supplier');
        });

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
                $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        });

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });

        // Retrieve other accounts and set 'table_name' to the fully-qualified model class name
        $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        // Concatenate all collections into a single collection
        $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);

        $items = ItemName::where('branch_id', session()->get('selected_branch'))->get();

        $tnc = AdminTermsAndCondition::where(['branch_id'=>session()->get('selected_branch'), 'type'=>'delivery_note'])->first();
        $deliveryNote = DeliveryNote::findOrFail(request()->delivery_note);
        $deliveryNote = DeliveryNote::with('account', 'item')->where(['branch_id'=>session()->get('selected_branch'), 'do_no'=>$deliveryNote->do_no])->get();
        // dd($deliveryNote);

        return view('crusher.trans_delivery_note.edit', compact('tnc','customers', 'items','accounts', 'deliveryNote'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Quotation  $quotation
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $deliveryNote)
    {
        try {
            $validatedData = $request->validate([
                'quotation_no' => 'nullable|string',
                'do_no' => 'nullable|string',
                'date' => 'nullable|date',
                'account_id' => 'nullable',
                'account_type' => 'nullable',
                'item_id.*' => 'nullable|exists:item_names,id',
                'item_description.*' => 'nullable|string',
                'quantity.*' => 'nullable|numeric',
                'total_quantity' => 'nullable|numeric',
                'narrations' => 'nullable|string',
            ]);

            $branch_id = session()->get('selected_branch');

            // If no quotation_no is provided, create delivery notes for each item
            $branch_id = session()->get('selected_branch');
            $deliveryNote = DeliveryNote::findOrFail(request()->delivery_note);
            $deliveryNote = DeliveryNote::with('account', 'item')->where(['branch_id'=>session()->get('selected_branch'), 'do_no'=>$deliveryNote->do_no])->forceDelete();

            for ($i = 0; $i < count($validatedData['quantity']); $i++) {
                $deliveryNoteData = [
                    'do_no' => $validatedData['do_no'],
                    'total_quantity' => $validatedData['total_quantity'],
                    'narrations' => $validatedData['narrations'],
                    'quotation_no' => $validatedData['quotation_no'], // Use URL if available, otherwise use form input
                    'date' => $validatedData['date'],
                    'account_id' => $validatedData['account_id'],
                    'account_type' => $validatedData['account_type'],
                    'item_id' => $validatedData['item_id'][$i],
                    'item_description' => $validatedData['item_description'][$i],
                    'quantity' => $validatedData['quantity'][$i],
                    'branch_id' => $branch_id,
                ];

                $do = DeliveryNote::create($deliveryNoteData);
            }

            flash()->addSuccess('Record Updated');
            return redirect()->route('crusher.delivery_notes.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Quotation  $quotation
     * @return \Illuminate\Http\Response
     */
    public function destroy(Quotation $quotation)
    {
        //
    }

    public function softDelete($deliveryNote)
    {
        $deliveryNote = DeliveryNote::findOrFail($deliveryNote);
        $deliveryNote->where('do_no',$deliveryNote->do_no)->delete();
        flash()->addSuccess('Success');
        return redirect()->route('crusher.delivery_notes.index');
    }
    function buyer(Request $request)
    {
        // return $request->all();
        try {
            unset($request['buyer_account_id']);
            unset($request['_token']);
            $request['type']='sales_invoice';
            OtherAccountBuyer::firstOrCreate(['type'=>'sales_invoice', 'invoice_no'=>$request->invoice_no],$request->all());
            return response()->json(['status'=>true]);
        } catch (\Throwable $th) {
            return response()->json(['status'=>false, 'error'=>$th->getMessage()]);
        }
    }

    function buyerExists(Request $request)
    {
        // return $request->all();
        try {
            $buyer = OtherAccountBuyer::where(['type'=>'sales_invoice', 'invoice_no'=>$request->invoice_no])->exists();
            return response()->json(['status'=>$buyer]);
        } catch (\Throwable $th) {
            return response()->json(['status'=>false, 'error'=>$th->getMessage()]);
        }
    }

    function buyerDelete(Request $request)
    {
        // return $request->all();
        try {
            $buyer = OtherAccountBuyer::where(['type'=>'sales_invoice', 'invoice_no'=>$request->invoice_no])->delete();
            return response()->json(['status'=>$buyer]);
        } catch (\Throwable $th) {
            return response()->json(['status'=>false, 'error'=>$th->getMessage()]);
        }
    }
}
